@@ -8,8 +8,9 @@ module Data.Monoid.SemiDirectProduct
        ) where
 
 #if !MIN_VERSION_base(4,8,0)
-import           Data.Monoid
+import           Data.Monoid        (Monoid(..))
 #endif
+import           Data.Semigroup     (Semigroup(..))
 
 import           Data.Monoid.Action
 
@@ -29,16 +30,27 @@ import           Data.Monoid.Action
 --   quotient.
 newtype Semi s m = Semi { unSemi :: (s,m) }
 
+instance (Semigroup m, Semigroup s, Action m s) => Semigroup (Semi s m) where
+  x <> y = Semi (xs <> (xm `act` ys), xm <> ym)
+    where (xs, xm) = unSemi x
+          (ys, ym) = unSemi y
+  {-# INLINE (<>) #-}
+
+  sconcat = foldr1 (<>)
+  {-# INLINE sconcat #-}
 
 instance (Monoid m, Monoid s, Action m s) => Monoid (Semi s m) where
   mempty      = Semi (mempty, mempty)
   {-# INLINE mempty #-}
 
+#if !MIN_VERSION_base(4,11,0)
   mappend x y = Semi (xs `mappend` (xm `act` ys), xm `mappend` ym)
     where (xs, xm) = unSemi x
           (ys, ym) = unSemi y
 
   {-# INLINE mappend #-}
+#endif
+
   mconcat     = foldr mappend mempty
   {-# INLINE mconcat #-}
 
@@ -47,7 +59,7 @@ instance (Monoid m, Monoid s, Action m s) => Monoid (Semi s m) where
 tag :: s -> m -> Semi s m
 tag s m = Semi (s,m)
 
--- | The injection map, *i.e.* give an @s@ value a trivial tag.
+-- | The injection map, /i.e./ give an @s@ value a trivial tag.
 inject :: Monoid m => s -> Semi s m
 inject = Semi . (,mempty)
 
@@ -62,7 +74,7 @@ untag = fst . unSemi
 --
 --   and
 --
---   @embed m <> inject s@ = tag (act m s) m@.
+--   @embed m <> inject s@ = tag (act m s) m@
 --
 --   The semi-direct product gives a split extension of @s@ by
 --   @m@. This allows us to embed @m@ into the semi-direct
@@ -71,7 +83,7 @@ untag = fst . unSemi
 embed :: Monoid s => m -> Semi s m
 embed = Semi . (mempty,)
 
--- | The quotient map, *i.e.* retrieve the monoidal tag value.
+-- | The quotient map, /i.e./ retrieve the monoidal tag value.
 quotient :: Semi s m -> m
 quotient = snd . unSemi
 
